"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultEventDecoder = exports.StandardEventDecoder = void 0;
exports.registerDefaultDecoder = registerDefaultDecoder;
// Copyright (C) 2023 Gnuxie <Gnuxie@protonmail.com>
//
// SPDX-License-Identifier: AFL-3.0
const typebox_1 = require("@sinclair/typebox");
const Action_1 = require("../Interface/Action");
const Value_1 = require("../Interface/Value");
const Events_1 = require("./Events");
const immutable_1 = require("immutable");
class StandardEventDecoder {
    constructor(decodersByType = (0, immutable_1.Map)()) {
        this.decodersByType = decodersByType;
        // nothing to do.
    }
    static blankEventDecoder() {
        return new StandardEventDecoder((0, immutable_1.Map)());
    }
    getDecoderForEventType(type) {
        return this.decodersByType.get(type);
    }
    setDecoderForEventType(type, decoder) {
        return new StandardEventDecoder(this.decodersByType.set(type, decoder));
    }
    decodeEvent(event) {
        if (event === null ||
            typeof event !== 'object' ||
            !('type' in event) ||
            typeof event['type'] !== 'string') {
            throw new TypeError(`Somehow there's malformed events being given by the homeserver.`);
        }
        const decoder = this.decodersByType.get(event.type);
        if (decoder === undefined) {
            return Value_1.Value.Decode(UnknownEvent, event);
        }
        else {
            return decoder(event);
        }
    }
    decodeStateEvent(event) {
        const result = this.decodeEvent(event);
        if ((0, Action_1.isError)(result)) {
            return result;
        }
        else if ('state_key' in result.ok &&
            typeof result.ok.state_key === 'string') {
            return result;
        }
        throw new TypeError('Somehow decoded a state event without a state key');
    }
}
exports.StandardEventDecoder = StandardEventDecoder;
const UnknownEvent = (0, Events_1.RoomEvent)(typebox_1.Type.Unknown());
exports.DefaultEventDecoder = StandardEventDecoder.blankEventDecoder();
function registerDefaultDecoder(type, decoder) {
    exports.DefaultEventDecoder = exports.DefaultEventDecoder.setDecoderForEventType(type, decoder);
}
//# sourceMappingURL=EventDecoder.js.map